/*
 * domain_stats.c: domain stats extraction helpers
 *
 * Copyright (C) 2006-2016 Red Hat, Inc.
 * Copyright (C) 2006-2008 Daniel P. Berrange
 * Copyright (c) 2018 SUSE LINUX Products GmbH, Nuernberg, Germany.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Author: Daniel P. Berrange <berrange@redhat.com>
 */

#include <config.h>

#include <stdio.h>

#include "virlog.h"
#include "domain_stats.h"
#include "virtypedparam.h"
#include "virnetdevtap.h"
#include "virnetdevopenvswitch.h"

#define VIR_FROM_THIS VIR_FROM_DOMAIN

VIR_LOG_INIT("conf.domain_stats");

int
virDomainStatsGetState(virDomainObjPtr dom,
                       virTypedParamListPtr params)
{
    if (virTypedParamListAddInt(params, dom->state.state, "state.state") < 0)
        return -1;

    if (virTypedParamListAddInt(params, dom->state.reason, "state.reason") < 0)
        return -1;

    return 0;
}

#define STATS_ADD_NET_PARAM(params, num, name, value) \
    if (value >= 0 && \
        virTypedParamListAddULLong((params), (value), "net.%zu.%s", (num), (name)) < 0) \
        return -1;

int
virDomainStatsGetInterface(virDomainObjPtr dom,
                           virTypedParamListPtr params)
{
    size_t i;
    struct _virDomainInterfaceStats tmp;

    if (!virDomainObjIsActive(dom))
        return 0;

    if (virTypedParamListAddUInt(params, dom->def->nnets, "net.count") < 0)
        return -1;

    /* Check the path is one of the domain's network interfaces. */
    for (i = 0; i < dom->def->nnets; i++) {
        virDomainNetDefPtr net = dom->def->nets[i];
        virDomainNetType actualType;

        if (!net->ifname)
            continue;

        memset(&tmp, 0, sizeof(tmp));

        actualType = virDomainNetGetActualType(net);

        if (virTypedParamListAddString(params, net->ifname, "net.%zu.name", i) < 0)
            return -1;

        if (actualType == VIR_DOMAIN_NET_TYPE_VHOSTUSER) {
            if (virNetDevOpenvswitchInterfaceStats(net->ifname, &tmp) < 0) {
                virResetLastError();
                continue;
            }
        } else {
            if (virNetDevTapInterfaceStats(net->ifname, &tmp,
                                           !virDomainNetTypeSharesHostView(net)) < 0) {
                virResetLastError();
                continue;
            }
        }

        STATS_ADD_NET_PARAM(params, i,
                            "rx.bytes", tmp.rx_bytes);
        STATS_ADD_NET_PARAM(params, i,
                            "rx.pkts", tmp.rx_packets);
        STATS_ADD_NET_PARAM(params, i,
                            "rx.errs", tmp.rx_errs);
        STATS_ADD_NET_PARAM(params, i,
                            "rx.drop", tmp.rx_drop);
        STATS_ADD_NET_PARAM(params, i,
                            "tx.bytes", tmp.tx_bytes);
        STATS_ADD_NET_PARAM(params, i,
                            "tx.pkts", tmp.tx_packets);
        STATS_ADD_NET_PARAM(params, i,
                            "tx.errs", tmp.tx_errs);
        STATS_ADD_NET_PARAM(params, i,
                            "tx.drop", tmp.tx_drop);
    }

    return 0;
}

#undef STATS_ADD_NET_PARAM
